% Generate js code for activations and gradients (statistical measures)
fileID = fopen('code.js', 'w');
writeCode(fileID, 'Activations')
writeCode(fileID, 'Gradients')
fclose(fileID);

%%
function [] = writeCode(fileID, type)
    %% Initialization
    run('config.m');

    %% Generate code
    fprintf(fileID, 'var data%s = [\n', type);
    for a = 1:length(activations)
        [WMatrices, UMatrices, YMatrices] = forward(nLayers, activations{a}.trans);
        [gradients] = backprop(WMatrices, UMatrices, YMatrices, activations{a}.transDiff);

        strRange = '[';
        strMean = '[';
        for i = 1:nLayers
            if strcmp(type, 'Activations')
                dataMiddle = mean(YMatrices{i}(:));
                
                dataStd = std(YMatrices{i}(:));
                dataTop = dataMiddle + dataStd;
                dataBottom = dataMiddle - dataStd;
                
%                 dataTop = dataMiddle + max(YMatrices{i}(:));
%                 dataBottom = dataMiddle + min(YMatrices{i}(:));
            elseif strcmp(type, 'Gradients')
                dataMiddle = mean(gradients{i}(:));
                
                dataStd = std(gradients{i}(:));
                dataTop = dataMiddle + dataStd;
                dataBottom = dataMiddle - dataStd;
                
%                 dataTop = dataMiddle + max(gradients{i}(:));
%                 dataBottom = dataMiddle + min(gradients{i}(:));
            end

            strMean = [strMean, sprintf('[%d, %f]', i, dataMiddle)];
            strRange = [strRange, sprintf('[%d, %f, %f]', i, dataBottom, dataTop)];

            if i < nLayers
                strRange = [strRange, ','];
                strMean = [strMean, ','];
            end
        end
        strRange = [strRange, ']'];
        strMean = [strMean, ']'];

        fprintf(fileID, '\t{\n\t\tname: "%s",\n\t\tmean: %s,\n\t\trange: %s\n\t}', activations{a}.name, strMean, strRange);

        if a < length(activations)
            fprintf(fileID, ',');
        end

        fprintf(fileID, '\n');
    end

    fprintf(fileID, '];\n');
end
