function [gradients] = backprop(W, U, Y, transDiff)
    assert(iscell(W) && iscell(U) && iscell(Y), 'All three matrices vectors must be cell arrays');
    assert(length(W) == length(U) && length(W) == length(Y), 'All three matrices vectors must have the same length');
    assert(isa(transDiff, 'function_handle'), 'transDiff should be defined as an anonymous function');
    
    rng(1337, 'combRecursive');
    nLayers = length(Y);
    
    % The teaching signal is generated randomly (this is, of course, not realistic but sufficient for the simulation)
    T = randn(size(Y{nLayers}, 1), size(Y{nLayers}, 2));
    
    % Calculate the errors from the back to the front
    deltas = cell(1, nLayers);
    deltas{nLayers} = ((T - Y{nLayers}) .* transDiff(U{nLayers}))';
    
    for i = nLayers-1:-1:1
        deltas{i} = W{i+1} * deltas{i+1} .* transDiff(U{i})';
    end
    
    % Store the gradient flowing from the back to the front
    gradients = cell(1, nLayers);
    
    for i = nLayers:-1:1
        gradients{i} = deltas{i}' .* Y{i};
    end
end
